/*
 * impost-sled.c - tool for generating NOP sleds
 * 
 * Written by ziplock
 * 
 * Comments: sickbeatz@hotmail.com IRC: #b4b0 on EFNet
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "getopt.h"

#define ARCH_X86   1
#define ARCH_SPARC 2

/* Table containing valid x86 nop code instructions  */
const char      x86_nop_list[] =
{
	0x27, 0x2f, 0x33, 0x37, 0x3f, 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
	0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52,
	0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
	0x60, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a,
	0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xf5, 0xf7, 0xf8, 0xf9, 0xfc
};

/* Table containing SPARC nop code instructions  */
unsigned long   sparc_nop_list[16] =
{
	0x20bfbf00, 0x801bc00f, 0x89a50822, 0x983e8012, 0xa2034012,
	0xa20e8013, 0xa21a400a, 0xa21c8012, 0xa4274012, 0xa61cc013,
	0xac15a16e, 0xb2268019, 0xb606401a, 0xfffffff8, 0xfffffff8
};

/*
 * Get a random integer between 0 and `highest' We'll use the a single byte
 * read from /dev/urandom as a seed
 */
int
get_random(highest)
	int             highest;
{
	int             urandom;
	unsigned char   seed = 0;

	/* Open /dev/urandom for reading  */
	urandom = open("/dev/urandom", O_RDONLY);
	if (urandom < 0)
	{
		/* Show error and exit 1  */
		perror("open()");
		exit(1);
	}
	do
	{
		read(urandom, &seed, 1);
	}
	while (!seed);
	if (!seed)
	{
		/* Unable to read from urandom?  */
		perror("read()");
		exit(1);
	}
	close(urandom);

	/* Set the random sequence seed  */
	srand(seed);

	/* Return  */
	return (1 + (rand() % highest));
}

/* Display help message  */
void
usage(appname)
	char           *appname;
{
	printf("Usage: %s [OPTION]\n"
	       "Options:\n"
	       "  -a, --arch=TYPE  System type\n"
	       "                   1=x86, 2=sparc\n\n"
	       "  -s, --size=SIZE  Size of NOP sled to generate\n"
	"          (multiply by 4 to get size in bytes for SPARC sleds)\n\n"
	       "  -t, --type=TYPE  Type of NOP sled to generate\n"
	       "\nNOP sled types:\n"
	       "  1  Single repeating operation code\n"
	       "  2  Random operation codes\n"
	       "\nReport bugs to sickbeatz@hotmail.com\n", appname);

	return;
}

/* Main program function  */
int
main(argc, argv)
	int             argc;
	char           *argv[];
{
	int             c, i, list_size, x = 0;
	int             arch_type = 0, sled_type = 0, sled_size = 0;

	extern char    *optarg;
	char           *sps = (char *)malloc(9216);

	do
	{
		/* Acceptable options  */
		static struct option long_options[] =
		{
			{"arch", required_argument, 0, 'a'},
			{"size", required_argument, 0, 's'},
			{"type", required_argument, 0, 't'},
			{"version", no_argument, 0, 'v'},
			{"help", no_argument, 0, 'h'},
			{0, 0, 0, 0}
		};
		int             option_index = 0;

		if ((c = getopt_long(argc, argv, "a:s:t:h?", long_options,
				     &option_index)) == -1)
			break;

		/* Handle each option  */
		switch (c)
		{
			/* System type  */
		case 'a':
			switch (atoi(optarg))
			{
			case 1:
				arch_type = ARCH_X86;
				break;
			case 2:
				arch_type = ARCH_SPARC;
				break;
			default:
				fprintf(stderr, "Error: Unknown system type %d\n",
					atoi(optarg));
				return 1;
			}
			break;

			/* Sled size  */
		case 's':
			sled_size = atoi(optarg);
			break;

			/* Sled type  */
		case 't':
			sled_type = atoi(optarg);
			if (!sled_type || sled_type > 3)
			{
				fprintf(stderr, "Error: option `--type' - unknown sled type\n");
				return 1;
			}
			break;
		default:
			usage(argv[0]);
			return 1;
		}
	}
	while (1);

	/* We're missing an argument  */
	if (arch_type == 0 || sled_type == 0 || sled_size == 0)
	{
		usage(argv[0]);
		return 1;
	}
	/* Create NOP sled for x86  */
	if (arch_type == ARCH_X86)
	{
		switch (sled_type)
		{
			/* Generic  */
		case 1:
			for (i = 0; i < sled_size; i++)
				putchar(0x90);
			return 0;
			break;
			/* Random  */
		case 2:
			for (i = 0; i < sled_size; i++)
			{
				list_size = strlen(x86_nop_list);
				putchar(x86_nop_list[get_random(list_size)]);
			}
			return 0;
		default:
			return 1;
		}
	}
	/* Create NOP sled for SPARC  */
	if (arch_type == ARCH_SPARC)
	{
		memset(sps, 0, 9216);
		switch (sled_type)
		{
			/* Repeating  */
		case 1:
			c = get_random(16);
			for (i = 0, x = 0; i < sled_size; i++)
			{
				sps[x] = (0x801bc00f >> 24) & 0xff;
				sps[x + 1] = (0x801bc00f >> 16) & 0xff;
				sps[x + 2] = (0x801bc00f >> 8) & 0xff;
				sps[x + 3] = (0x801bc00f >> 0) & 0xff;
				x += 4;
			}
			printf("%s", sps);
			return 0;

			/* Random  */
		case 2:
			for (i = 0, x = 0; i < sled_size; i++)
			{
				c = get_random(16);
				if ((sparc_nop_list[c] != 0) &&
				    (sparc_nop_list[c] != 0xfffffff8) &&
				    c < 15)
				{
					sps[x] = (sparc_nop_list[c] >> 24) & 0xff;
					sps[x + 1] = (sparc_nop_list[c] >> 16) & 0xff;
					sps[x + 2] = (sparc_nop_list[c] >> 8) & 0xff;
					sps[x + 3] = (sparc_nop_list[c] >> 0) & 0xff;
					x += 4;
				}
			}
			printf("%s", sps);
			return 0;
		}
	}
	usage(argv[1]);
	return 1;
}

/* EOF */
